# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["FxGetRatesParams"]


class FxGetRatesParams(TypedDict, total=False):
    base_currency: Annotated[str, PropertyInfo(alias="baseCurrency")]
    """The base currency code (e.g., USD)."""

    forecast_days: Annotated[int, PropertyInfo(alias="forecastDays")]
    """Number of days into the future to provide an AI-driven prediction."""

    target_currency: Annotated[str, PropertyInfo(alias="targetCurrency")]
    """The target currency code (e.g., EUR)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options

__all__ = ["OffersResource", "AsyncOffersResource"]


class OffersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> OffersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/diplomat-bit/aibank#accessing-raw-response-data-eg-headers
        """
        return OffersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OffersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/diplomat-bit/aibank#with_streaming_response
        """
        return OffersResourceWithStreamingResponse(self)

    def redeem(
        self,
        offer_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Redeems a personalized, exclusive offer from the Plato AI marketplace, often
        resulting in a discount, special rate, or credit to the user's account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not offer_id:
            raise ValueError(f"Expected a non-empty value for `offer_id` but received {offer_id!r}")
        return self._post(
            f"/marketplace/offers/{offer_id}/redeem",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncOffersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncOffersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/diplomat-bit/aibank#accessing-raw-response-data-eg-headers
        """
        return AsyncOffersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOffersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/diplomat-bit/aibank#with_streaming_response
        """
        return AsyncOffersResourceWithStreamingResponse(self)

    async def redeem(
        self,
        offer_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Redeems a personalized, exclusive offer from the Plato AI marketplace, often
        resulting in a discount, special rate, or credit to the user's account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not offer_id:
            raise ValueError(f"Expected a non-empty value for `offer_id` but received {offer_id!r}")
        return await self._post(
            f"/marketplace/offers/{offer_id}/redeem",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class OffersResourceWithRawResponse:
    def __init__(self, offers: OffersResource) -> None:
        self._offers = offers

        self.redeem = to_raw_response_wrapper(
            offers.redeem,
        )


class AsyncOffersResourceWithRawResponse:
    def __init__(self, offers: AsyncOffersResource) -> None:
        self._offers = offers

        self.redeem = async_to_raw_response_wrapper(
            offers.redeem,
        )


class OffersResourceWithStreamingResponse:
    def __init__(self, offers: OffersResource) -> None:
        self._offers = offers

        self.redeem = to_streamed_response_wrapper(
            offers.redeem,
        )


class AsyncOffersResourceWithStreamingResponse:
    def __init__(self, offers: AsyncOffersResource) -> None:
        self._offers = offers

        self.redeem = async_to_streamed_response_wrapper(
            offers.redeem,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .audits import (
    AuditsResource,
    AsyncAuditsResource,
    AuditsResourceWithRawResponse,
    AsyncAuditsResourceWithRawResponse,
    AuditsResourceWithStreamingResponse,
    AsyncAuditsResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["ComplianceResource", "AsyncComplianceResource"]


class ComplianceResource(SyncAPIResource):
    @cached_property
    def audits(self) -> AuditsResource:
        return AuditsResource(self._client)

    @cached_property
    def with_raw_response(self) -> ComplianceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/diplomat-bit/aibank#accessing-raw-response-data-eg-headers
        """
        return ComplianceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ComplianceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/diplomat-bit/aibank#with_streaming_response
        """
        return ComplianceResourceWithStreamingResponse(self)


class AsyncComplianceResource(AsyncAPIResource):
    @cached_property
    def audits(self) -> AsyncAuditsResource:
        return AsyncAuditsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncComplianceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/diplomat-bit/aibank#accessing-raw-response-data-eg-headers
        """
        return AsyncComplianceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncComplianceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/diplomat-bit/aibank#with_streaming_response
        """
        return AsyncComplianceResourceWithStreamingResponse(self)


class ComplianceResourceWithRawResponse:
    def __init__(self, compliance: ComplianceResource) -> None:
        self._compliance = compliance

    @cached_property
    def audits(self) -> AuditsResourceWithRawResponse:
        return AuditsResourceWithRawResponse(self._compliance.audits)


class AsyncComplianceResourceWithRawResponse:
    def __init__(self, compliance: AsyncComplianceResource) -> None:
        self._compliance = compliance

    @cached_property
    def audits(self) -> AsyncAuditsResourceWithRawResponse:
        return AsyncAuditsResourceWithRawResponse(self._compliance.audits)


class ComplianceResourceWithStreamingResponse:
    def __init__(self, compliance: ComplianceResource) -> None:
        self._compliance = compliance

    @cached_property
    def audits(self) -> AuditsResourceWithStreamingResponse:
        return AuditsResourceWithStreamingResponse(self._compliance.audits)


class AsyncComplianceResourceWithStreamingResponse:
    def __init__(self, compliance: AsyncComplianceResource) -> None:
        self._compliance = compliance

    @cached_property
    def audits(self) -> AsyncAuditsResourceWithStreamingResponse:
        return AsyncAuditsResourceWithStreamingResponse(self._compliance.audits)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...types import account_list_params
from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from .overdraft import (
    OverdraftResource,
    AsyncOverdraftResource,
    OverdraftResourceWithRawResponse,
    AsyncOverdraftResourceWithRawResponse,
    OverdraftResourceWithStreamingResponse,
    AsyncOverdraftResourceWithStreamingResponse,
)
from .statements import (
    StatementsResource,
    AsyncStatementsResource,
    StatementsResourceWithRawResponse,
    AsyncStatementsResourceWithRawResponse,
    StatementsResourceWithStreamingResponse,
    AsyncStatementsResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .transactions import (
    TransactionsResource,
    AsyncTransactionsResource,
    TransactionsResourceWithRawResponse,
    AsyncTransactionsResourceWithRawResponse,
    TransactionsResourceWithStreamingResponse,
    AsyncTransactionsResourceWithStreamingResponse,
)
from ..._base_client import make_request_options
from ...types.account_retrieve_response import AccountRetrieveResponse

__all__ = ["AccountsResource", "AsyncAccountsResource"]


class AccountsResource(SyncAPIResource):
    @cached_property
    def transactions(self) -> TransactionsResource:
        return TransactionsResource(self._client)

    @cached_property
    def statements(self) -> StatementsResource:
        return StatementsResource(self._client)

    @cached_property
    def overdraft(self) -> OverdraftResource:
        return OverdraftResource(self._client)

    @cached_property
    def with_raw_response(self) -> AccountsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/diplomat-bit/aibank#accessing-raw-response-data-eg-headers
        """
        return AccountsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AccountsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/diplomat-bit/aibank#with_streaming_response
        """
        return AccountsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        account_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountRetrieveResponse:
        """
        Retrieves comprehensive analytics for a specific financial account, including
        historical balance trends, projected cash flow, and AI-driven insights into
        spending patterns.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            f"/accounts/{account_id}/details",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountRetrieveResponse,
        )

    def list(
        self,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Fetches a comprehensive, real-time list of all external financial accounts
        linked to the user's profile, including consolidated balances and institutional
        details.

        Args:
          limit: Maximum number of items to return in a single page.

          offset: Number of items to skip before starting to collect the result set.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/accounts/me",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    account_list_params.AccountListParams,
                ),
            ),
            cast_to=object,
        )

    def link(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Begins the secure process of linking a new external financial institution (e.g.,
        another bank, investment platform) to the user's profile, typically involving a
        third-party tokenized flow.
        """
        return self._post(
            "/accounts/link",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncAccountsResource(AsyncAPIResource):
    @cached_property
    def transactions(self) -> AsyncTransactionsResource:
        return AsyncTransactionsResource(self._client)

    @cached_property
    def statements(self) -> AsyncStatementsResource:
        return AsyncStatementsResource(self._client)

    @cached_property
    def overdraft(self) -> AsyncOverdraftResource:
        return AsyncOverdraftResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncAccountsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/diplomat-bit/aibank#accessing-raw-response-data-eg-headers
        """
        return AsyncAccountsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAccountsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/diplomat-bit/aibank#with_streaming_response
        """
        return AsyncAccountsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        account_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountRetrieveResponse:
        """
        Retrieves comprehensive analytics for a specific financial account, including
        historical balance trends, projected cash flow, and AI-driven insights into
        spending patterns.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            f"/accounts/{account_id}/details",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccountRetrieveResponse,
        )

    async def list(
        self,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Fetches a comprehensive, real-time list of all external financial accounts
        linked to the user's profile, including consolidated balances and institutional
        details.

        Args:
          limit: Maximum number of items to return in a single page.

          offset: Number of items to skip before starting to collect the result set.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/accounts/me",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    account_list_params.AccountListParams,
                ),
            ),
            cast_to=object,
        )

    async def link(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Begins the secure process of linking a new external financial institution (e.g.,
        another bank, investment platform) to the user's profile, typically involving a
        third-party tokenized flow.
        """
        return await self._post(
            "/accounts/link",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AccountsResourceWithRawResponse:
    def __init__(self, accounts: AccountsResource) -> None:
        self._accounts = accounts

        self.retrieve = to_raw_response_wrapper(
            accounts.retrieve,
        )
        self.list = to_raw_response_wrapper(
            accounts.list,
        )
        self.link = to_raw_response_wrapper(
            accounts.link,
        )

    @cached_property
    def transactions(self) -> TransactionsResourceWithRawResponse:
        return TransactionsResourceWithRawResponse(self._accounts.transactions)

    @cached_property
    def statements(self) -> StatementsResourceWithRawResponse:
        return StatementsResourceWithRawResponse(self._accounts.statements)

    @cached_property
    def overdraft(self) -> OverdraftResourceWithRawResponse:
        return OverdraftResourceWithRawResponse(self._accounts.overdraft)


class AsyncAccountsResourceWithRawResponse:
    def __init__(self, accounts: AsyncAccountsResource) -> None:
        self._accounts = accounts

        self.retrieve = async_to_raw_response_wrapper(
            accounts.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            accounts.list,
        )
        self.link = async_to_raw_response_wrapper(
            accounts.link,
        )

    @cached_property
    def transactions(self) -> AsyncTransactionsResourceWithRawResponse:
        return AsyncTransactionsResourceWithRawResponse(self._accounts.transactions)

    @cached_property
    def statements(self) -> AsyncStatementsResourceWithRawResponse:
        return AsyncStatementsResourceWithRawResponse(self._accounts.statements)

    @cached_property
    def overdraft(self) -> AsyncOverdraftResourceWithRawResponse:
        return AsyncOverdraftResourceWithRawResponse(self._accounts.overdraft)


class AccountsResourceWithStreamingResponse:
    def __init__(self, accounts: AccountsResource) -> None:
        self._accounts = accounts

        self.retrieve = to_streamed_response_wrapper(
            accounts.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            accounts.list,
        )
        self.link = to_streamed_response_wrapper(
            accounts.link,
        )

    @cached_property
    def transactions(self) -> TransactionsResourceWithStreamingResponse:
        return TransactionsResourceWithStreamingResponse(self._accounts.transactions)

    @cached_property
    def statements(self) -> StatementsResourceWithStreamingResponse:
        return StatementsResourceWithStreamingResponse(self._accounts.statements)

    @cached_property
    def overdraft(self) -> OverdraftResourceWithStreamingResponse:
        return OverdraftResourceWithStreamingResponse(self._accounts.overdraft)


class AsyncAccountsResourceWithStreamingResponse:
    def __init__(self, accounts: AsyncAccountsResource) -> None:
        self._accounts = accounts

        self.retrieve = async_to_streamed_response_wrapper(
            accounts.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            accounts.list,
        )
        self.link = async_to_streamed_response_wrapper(
            accounts.link,
        )

    @cached_property
    def transactions(self) -> AsyncTransactionsResourceWithStreamingResponse:
        return AsyncTransactionsResourceWithStreamingResponse(self._accounts.transactions)

    @cached_property
    def statements(self) -> AsyncStatementsResourceWithStreamingResponse:
        return AsyncStatementsResourceWithStreamingResponse(self._accounts.statements)

    @cached_property
    def overdraft(self) -> AsyncOverdraftResourceWithStreamingResponse:
        return AsyncOverdraftResourceWithStreamingResponse(self._accounts.overdraft)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from jocall3 import Jocall3, AsyncJocall3
from tests.utils import assert_matches_type
from jocall3.types.ai.oracle import SimulateRunStandardResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSimulate:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_run_advanced(self, client: Jocall3) -> None:
        simulate = client.ai.oracle.simulate.run_advanced()
        assert_matches_type(object, simulate, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_run_advanced_with_all_params(self, client: Jocall3) -> None:
        simulate = client.ai.oracle.simulate.run_advanced(
            global_economic_factors={},
            personal_assumptions={},
        )
        assert_matches_type(object, simulate, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_run_advanced(self, client: Jocall3) -> None:
        response = client.ai.oracle.simulate.with_raw_response.run_advanced()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        simulate = response.parse()
        assert_matches_type(object, simulate, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_run_advanced(self, client: Jocall3) -> None:
        with client.ai.oracle.simulate.with_streaming_response.run_advanced() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            simulate = response.parse()
            assert_matches_type(object, simulate, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_run_standard(self, client: Jocall3) -> None:
        simulate = client.ai.oracle.simulate.run_standard()
        assert_matches_type(SimulateRunStandardResponse, simulate, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_run_standard(self, client: Jocall3) -> None:
        response = client.ai.oracle.simulate.with_raw_response.run_standard()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        simulate = response.parse()
        assert_matches_type(SimulateRunStandardResponse, simulate, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_run_standard(self, client: Jocall3) -> None:
        with client.ai.oracle.simulate.with_streaming_response.run_standard() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            simulate = response.parse()
            assert_matches_type(SimulateRunStandardResponse, simulate, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncSimulate:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_run_advanced(self, async_client: AsyncJocall3) -> None:
        simulate = await async_client.ai.oracle.simulate.run_advanced()
        assert_matches_type(object, simulate, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_run_advanced_with_all_params(self, async_client: AsyncJocall3) -> None:
        simulate = await async_client.ai.oracle.simulate.run_advanced(
            global_economic_factors={},
            personal_assumptions={},
        )
        assert_matches_type(object, simulate, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_run_advanced(self, async_client: AsyncJocall3) -> None:
        response = await async_client.ai.oracle.simulate.with_raw_response.run_advanced()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        simulate = await response.parse()
        assert_matches_type(object, simulate, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_run_advanced(self, async_client: AsyncJocall3) -> None:
        async with async_client.ai.oracle.simulate.with_streaming_response.run_advanced() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            simulate = await response.parse()
            assert_matches_type(object, simulate, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_run_standard(self, async_client: AsyncJocall3) -> None:
        simulate = await async_client.ai.oracle.simulate.run_standard()
        assert_matches_type(SimulateRunStandardResponse, simulate, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_run_standard(self, async_client: AsyncJocall3) -> None:
        response = await async_client.ai.oracle.simulate.with_raw_response.run_standard()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        simulate = await response.parse()
        assert_matches_type(SimulateRunStandardResponse, simulate, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_run_standard(self, async_client: AsyncJocall3) -> None:
        async with async_client.ai.oracle.simulate.with_streaming_response.run_standard() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            simulate = await response.parse()
            assert_matches_type(SimulateRunStandardResponse, simulate, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .account_list_params import AccountListParams as AccountListParams
from .user_register_params import UserRegisterParams as UserRegisterParams
from .user_register_response import UserRegisterResponse as UserRegisterResponse
from .transaction_list_params import TransactionListParams as TransactionListParams
from .account_retrieve_response import AccountRetrieveResponse as AccountRetrieveResponse
from .transaction_retrieve_response import TransactionRetrieveResponse as TransactionRetrieveResponse
from .transaction_categorize_response import TransactionCategorizeResponse as TransactionCategorizeResponse
from .marketplace_list_products_params import MarketplaceListProductsParams as MarketplaceListProductsParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .controls import (
    ControlsResource,
    AsyncControlsResource,
    ControlsResourceWithRawResponse,
    AsyncControlsResourceWithRawResponse,
    ControlsResourceWithStreamingResponse,
    AsyncControlsResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.corporate import card_list_params, card_issue_virtual_params
from ....types.corporate.card_freeze_response import CardFreezeResponse
from ....types.corporate.card_issue_virtual_response import CardIssueVirtualResponse

__all__ = ["CardsResource", "AsyncCardsResource"]


class CardsResource(SyncAPIResource):
    @cached_property
    def controls(self) -> ControlsResource:
        return ControlsResource(self._client)

    @cached_property
    def with_raw_response(self) -> CardsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/diplomat-bit/aibank#accessing-raw-response-data-eg-headers
        """
        return CardsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CardsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/diplomat-bit/aibank#with_streaming_response
        """
        return CardsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Retrieves a comprehensive list of all physical and virtual corporate cards
        associated with the user's organization, including their status, assigned
        holder, and current spending controls.

        Args:
          limit: Maximum number of items to return in a single page.

          offset: Number of items to skip before starting to collect the result set.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/corporate/cards",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    card_list_params.CardListParams,
                ),
            ),
            cast_to=object,
        )

    def freeze(
        self,
        card_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CardFreezeResponse:
        """
        Immediately changes the frozen status of a corporate card, preventing or
        allowing transactions in real-time, critical for security and expense
        management.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not card_id:
            raise ValueError(f"Expected a non-empty value for `card_id` but received {card_id!r}")
        return self._post(
            f"/corporate/cards/{card_id}/freeze",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CardFreezeResponse,
        )

    def issue_virtual(
        self,
        *,
        controls: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CardIssueVirtualResponse:
        """
        Creates and issues a new virtual corporate card with specified spending limits,
        merchant restrictions, and expiration dates, ideal for secure online purchases
        and temporary projects.

        Args:
          controls: Granular spending controls for a corporate card.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/corporate/cards/virtual",
            body=maybe_transform({"controls": controls}, card_issue_virtual_params.CardIssueVirtualParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CardIssueVirtualResponse,
        )


class AsyncCardsResource(AsyncAPIResource):
    @cached_property
    def controls(self) -> AsyncControlsResource:
        return AsyncControlsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncCardsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/diplomat-bit/aibank#accessing-raw-response-data-eg-headers
        """
        return AsyncCardsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCardsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/diplomat-bit/aibank#with_streaming_response
        """
        return AsyncCardsResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Retrieves a comprehensive list of all physical and virtual corporate cards
        associated with the user's organization, including their status, assigned
        holder, and current spending controls.

        Args:
          limit: Maximum number of items to return in a single page.

          offset: Number of items to skip before starting to collect the result set.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/corporate/cards",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    card_list_params.CardListParams,
                ),
            ),
            cast_to=object,
        )

    async def freeze(
        self,
        card_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CardFreezeResponse:
        """
        Immediately changes the frozen status of a corporate card, preventing or
        allowing transactions in real-time, critical for security and expense
        management.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not card_id:
            raise ValueError(f"Expected a non-empty value for `card_id` but received {card_id!r}")
        return await self._post(
            f"/corporate/cards/{card_id}/freeze",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CardFreezeResponse,
        )

    async def issue_virtual(
        self,
        *,
        controls: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CardIssueVirtualResponse:
        """
        Creates and issues a new virtual corporate card with specified spending limits,
        merchant restrictions, and expiration dates, ideal for secure online purchases
        and temporary projects.

        Args:
          controls: Granular spending controls for a corporate card.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/corporate/cards/virtual",
            body=await async_maybe_transform({"controls": controls}, card_issue_virtual_params.CardIssueVirtualParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CardIssueVirtualResponse,
        )


class CardsResourceWithRawResponse:
    def __init__(self, cards: CardsResource) -> None:
        self._cards = cards

        self.list = to_raw_response_wrapper(
            cards.list,
        )
        self.freeze = to_raw_response_wrapper(
            cards.freeze,
        )
        self.issue_virtual = to_raw_response_wrapper(
            cards.issue_virtual,
        )

    @cached_property
    def controls(self) -> ControlsResourceWithRawResponse:
        return ControlsResourceWithRawResponse(self._cards.controls)


class AsyncCardsResourceWithRawResponse:
    def __init__(self, cards: AsyncCardsResource) -> None:
        self._cards = cards

        self.list = async_to_raw_response_wrapper(
            cards.list,
        )
        self.freeze = async_to_raw_response_wrapper(
            cards.freeze,
        )
        self.issue_virtual = async_to_raw_response_wrapper(
            cards.issue_virtual,
        )

    @cached_property
    def controls(self) -> AsyncControlsResourceWithRawResponse:
        return AsyncControlsResourceWithRawResponse(self._cards.controls)


class CardsResourceWithStreamingResponse:
    def __init__(self, cards: CardsResource) -> None:
        self._cards = cards

        self.list = to_streamed_response_wrapper(
            cards.list,
        )
        self.freeze = to_streamed_response_wrapper(
            cards.freeze,
        )
        self.issue_virtual = to_streamed_response_wrapper(
            cards.issue_virtual,
        )

    @cached_property
    def controls(self) -> ControlsResourceWithStreamingResponse:
        return ControlsResourceWithStreamingResponse(self._cards.controls)


class AsyncCardsResourceWithStreamingResponse:
    def __init__(self, cards: AsyncCardsResource) -> None:
        self._cards = cards

        self.list = async_to_streamed_response_wrapper(
            cards.list,
        )
        self.freeze = async_to_streamed_response_wrapper(
            cards.freeze,
        )
        self.issue_virtual = async_to_streamed_response_wrapper(
            cards.issue_virtual,
        )

    @cached_property
    def controls(self) -> AsyncControlsResourceWithStreamingResponse:
        return AsyncControlsResourceWithStreamingResponse(self._cards.controls)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from jocall3 import Jocall3, AsyncJocall3
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestIncubator:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_generate_pitch(self, client: Jocall3) -> None:
        incubator = client.ai.incubator.generate_pitch(
            financial_projections={
                "seedRoundAmount": 2500000,
                "valuationPreMoney": 10000000,
                "projectionYears": 3,
                "revenueForecast": [500000, 2000000, 6000000],
                "profitabilityEstimate": "Achieve profitability within 18 months.",
            },
        )
        assert_matches_type(object, incubator, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_generate_pitch(self, client: Jocall3) -> None:
        response = client.ai.incubator.with_raw_response.generate_pitch(
            financial_projections={
                "seedRoundAmount": 2500000,
                "valuationPreMoney": 10000000,
                "projectionYears": 3,
                "revenueForecast": [500000, 2000000, 6000000],
                "profitabilityEstimate": "Achieve profitability within 18 months.",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        incubator = response.parse()
        assert_matches_type(object, incubator, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_generate_pitch(self, client: Jocall3) -> None:
        with client.ai.incubator.with_streaming_response.generate_pitch(
            financial_projections={
                "seedRoundAmount": 2500000,
                "valuationPreMoney": 10000000,
                "projectionYears": 3,
                "revenueForecast": [500000, 2000000, 6000000],
                "profitabilityEstimate": "Achieve profitability within 18 months.",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            incubator = response.parse()
            assert_matches_type(object, incubator, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncIncubator:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_generate_pitch(self, async_client: AsyncJocall3) -> None:
        incubator = await async_client.ai.incubator.generate_pitch(
            financial_projections={
                "seedRoundAmount": 2500000,
                "valuationPreMoney": 10000000,
                "projectionYears": 3,
                "revenueForecast": [500000, 2000000, 6000000],
                "profitabilityEstimate": "Achieve profitability within 18 months.",
            },
        )
        assert_matches_type(object, incubator, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_generate_pitch(self, async_client: AsyncJocall3) -> None:
        response = await async_client.ai.incubator.with_raw_response.generate_pitch(
            financial_projections={
                "seedRoundAmount": 2500000,
                "valuationPreMoney": 10000000,
                "projectionYears": 3,
                "revenueForecast": [500000, 2000000, 6000000],
                "profitabilityEstimate": "Achieve profitability within 18 months.",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        incubator = await response.parse()
        assert_matches_type(object, incubator, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_generate_pitch(self, async_client: AsyncJocall3) -> None:
        async with async_client.ai.incubator.with_streaming_response.generate_pitch(
            financial_projections={
                "seedRoundAmount": 2500000,
                "valuationPreMoney": 10000000,
                "projectionYears": 3,
                "revenueForecast": [500000, 2000000, 6000000],
                "profitabilityEstimate": "Achieve profitability within 18 months.",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            incubator = await response.parse()
            assert_matches_type(object, incubator, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .fraud_list_rules_params import FraudListRulesParams as FraudListRulesParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .sustainability import (
    SustainabilityResource,
    AsyncSustainabilityResource,
    SustainabilityResourceWithRawResponse,
    AsyncSustainabilityResourceWithRawResponse,
    SustainabilityResourceWithStreamingResponse,
    AsyncSustainabilityResourceWithStreamingResponse,
)

__all__ = [
    "SustainabilityResource",
    "AsyncSustainabilityResource",
    "SustainabilityResourceWithRawResponse",
    "AsyncSustainabilityResourceWithRawResponse",
    "SustainabilityResourceWithStreamingResponse",
    "AsyncSustainabilityResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .incubator import (
    IncubatorResource,
    AsyncIncubatorResource,
    IncubatorResourceWithRawResponse,
    AsyncIncubatorResourceWithRawResponse,
    IncubatorResourceWithStreamingResponse,
    AsyncIncubatorResourceWithStreamingResponse,
)

__all__ = [
    "IncubatorResource",
    "AsyncIncubatorResource",
    "IncubatorResourceWithRawResponse",
    "AsyncIncubatorResourceWithRawResponse",
    "IncubatorResourceWithStreamingResponse",
    "AsyncIncubatorResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["AdvisorChatParams"]


class AdvisorChatParams(TypedDict, total=False):
    function_response: Annotated[object, PropertyInfo(alias="functionResponse")]
    """
    Optional: The output from a tool function that the AI previously requested to be
    executed.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["ControlUpdateResponse"]


class ControlUpdateResponse(BaseModel):
    controls: object
    """Granular spending controls for a corporate card."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...types import transaction_list_params
from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from .insights import (
    InsightsResource,
    AsyncInsightsResource,
    InsightsResourceWithRawResponse,
    AsyncInsightsResourceWithRawResponse,
    InsightsResourceWithStreamingResponse,
    AsyncInsightsResourceWithStreamingResponse,
)
from ..._compat import cached_property
from .recurring import (
    RecurringResource,
    AsyncRecurringResource,
    RecurringResourceWithRawResponse,
    AsyncRecurringResourceWithRawResponse,
    RecurringResourceWithStreamingResponse,
    AsyncRecurringResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.transaction_retrieve_response import TransactionRetrieveResponse
from ...types.transaction_categorize_response import TransactionCategorizeResponse

__all__ = ["TransactionsResource", "AsyncTransactionsResource"]


class TransactionsResource(SyncAPIResource):
    @cached_property
    def recurring(self) -> RecurringResource:
        return RecurringResource(self._client)

    @cached_property
    def insights(self) -> InsightsResource:
        return InsightsResource(self._client)

    @cached_property
    def with_raw_response(self) -> TransactionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/diplomat-bit/aibank#accessing-raw-response-data-eg-headers
        """
        return TransactionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TransactionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/diplomat-bit/aibank#with_streaming_response
        """
        return TransactionsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        transaction_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TransactionRetrieveResponse:
        """
        Retrieves granular information for a single transaction by its unique ID,
        including AI categorization confidence, merchant details, and associated carbon
        footprint.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not transaction_id:
            raise ValueError(f"Expected a non-empty value for `transaction_id` but received {transaction_id!r}")
        return self._get(
            f"/transactions/{transaction_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionRetrieveResponse,
        )

    def list(
        self,
        *,
        category: str | Omit = omit,
        end_date: str | Omit = omit,
        limit: int | Omit = omit,
        max_amount: int | Omit = omit,
        min_amount: int | Omit = omit,
        offset: int | Omit = omit,
        search_query: str | Omit = omit,
        start_date: str | Omit = omit,
        type: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Retrieves a paginated list of the user's transactions, with extensive options
        for filtering by type, category, date range, amount, and intelligent AI-driven
        sorting and search capabilities.

        Args:
          category: Filter transactions by their AI-assigned or user-defined category.

          end_date: Retrieve transactions up to this date (inclusive).

          limit: Maximum number of items to return in a single page.

          max_amount: Filter for transactions with an amount less than or equal to this value.

          min_amount: Filter for transactions with an amount greater than or equal to this value.

          offset: Number of items to skip before starting to collect the result set.

          search_query: Free-text search across transaction descriptions, merchants, and notes.

          start_date: Retrieve transactions from this date (inclusive).

          type: Filter transactions by type (e.g., income, expense, transfer).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/transactions",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "category": category,
                        "end_date": end_date,
                        "limit": limit,
                        "max_amount": max_amount,
                        "min_amount": min_amount,
                        "offset": offset,
                        "search_query": search_query,
                        "start_date": start_date,
                        "type": type,
                    },
                    transaction_list_params.TransactionListParams,
                ),
            ),
            cast_to=object,
        )

    def categorize(
        self,
        transaction_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TransactionCategorizeResponse:
        """
        Allows the user to override or refine the AI's categorization for a transaction,
        improving future AI accuracy and personal financial reporting.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not transaction_id:
            raise ValueError(f"Expected a non-empty value for `transaction_id` but received {transaction_id!r}")
        return self._put(
            f"/transactions/{transaction_id}/categorize",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionCategorizeResponse,
        )


class AsyncTransactionsResource(AsyncAPIResource):
    @cached_property
    def recurring(self) -> AsyncRecurringResource:
        return AsyncRecurringResource(self._client)

    @cached_property
    def insights(self) -> AsyncInsightsResource:
        return AsyncInsightsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncTransactionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/diplomat-bit/aibank#accessing-raw-response-data-eg-headers
        """
        return AsyncTransactionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTransactionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/diplomat-bit/aibank#with_streaming_response
        """
        return AsyncTransactionsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        transaction_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TransactionRetrieveResponse:
        """
        Retrieves granular information for a single transaction by its unique ID,
        including AI categorization confidence, merchant details, and associated carbon
        footprint.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not transaction_id:
            raise ValueError(f"Expected a non-empty value for `transaction_id` but received {transaction_id!r}")
        return await self._get(
            f"/transactions/{transaction_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionRetrieveResponse,
        )

    async def list(
        self,
        *,
        category: str | Omit = omit,
        end_date: str | Omit = omit,
        limit: int | Omit = omit,
        max_amount: int | Omit = omit,
        min_amount: int | Omit = omit,
        offset: int | Omit = omit,
        search_query: str | Omit = omit,
        start_date: str | Omit = omit,
        type: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Retrieves a paginated list of the user's transactions, with extensive options
        for filtering by type, category, date range, amount, and intelligent AI-driven
        sorting and search capabilities.

        Args:
          category: Filter transactions by their AI-assigned or user-defined category.

          end_date: Retrieve transactions up to this date (inclusive).

          limit: Maximum number of items to return in a single page.

          max_amount: Filter for transactions with an amount less than or equal to this value.

          min_amount: Filter for transactions with an amount greater than or equal to this value.

          offset: Number of items to skip before starting to collect the result set.

          search_query: Free-text search across transaction descriptions, merchants, and notes.

          start_date: Retrieve transactions from this date (inclusive).

          type: Filter transactions by type (e.g., income, expense, transfer).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/transactions",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "category": category,
                        "end_date": end_date,
                        "limit": limit,
                        "max_amount": max_amount,
                        "min_amount": min_amount,
                        "offset": offset,
                        "search_query": search_query,
                        "start_date": start_date,
                        "type": type,
                    },
                    transaction_list_params.TransactionListParams,
                ),
            ),
            cast_to=object,
        )

    async def categorize(
        self,
        transaction_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TransactionCategorizeResponse:
        """
        Allows the user to override or refine the AI's categorization for a transaction,
        improving future AI accuracy and personal financial reporting.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not transaction_id:
            raise ValueError(f"Expected a non-empty value for `transaction_id` but received {transaction_id!r}")
        return await self._put(
            f"/transactions/{transaction_id}/categorize",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TransactionCategorizeResponse,
        )


class TransactionsResourceWithRawResponse:
    def __init__(self, transactions: TransactionsResource) -> None:
        self._transactions = transactions

        self.retrieve = to_raw_response_wrapper(
            transactions.retrieve,
        )
        self.list = to_raw_response_wrapper(
            transactions.list,
        )
        self.categorize = to_raw_response_wrapper(
            transactions.categorize,
        )

    @cached_property
    def recurring(self) -> RecurringResourceWithRawResponse:
        return RecurringResourceWithRawResponse(self._transactions.recurring)

    @cached_property
    def insights(self) -> InsightsResourceWithRawResponse:
        return InsightsResourceWithRawResponse(self._transactions.insights)


class AsyncTransactionsResourceWithRawResponse:
    def __init__(self, transactions: AsyncTransactionsResource) -> None:
        self._transactions = transactions

        self.retrieve = async_to_raw_response_wrapper(
            transactions.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            transactions.list,
        )
        self.categorize = async_to_raw_response_wrapper(
            transactions.categorize,
        )

    @cached_property
    def recurring(self) -> AsyncRecurringResourceWithRawResponse:
        return AsyncRecurringResourceWithRawResponse(self._transactions.recurring)

    @cached_property
    def insights(self) -> AsyncInsightsResourceWithRawResponse:
        return AsyncInsightsResourceWithRawResponse(self._transactions.insights)


class TransactionsResourceWithStreamingResponse:
    def __init__(self, transactions: TransactionsResource) -> None:
        self._transactions = transactions

        self.retrieve = to_streamed_response_wrapper(
            transactions.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            transactions.list,
        )
        self.categorize = to_streamed_response_wrapper(
            transactions.categorize,
        )

    @cached_property
    def recurring(self) -> RecurringResourceWithStreamingResponse:
        return RecurringResourceWithStreamingResponse(self._transactions.recurring)

    @cached_property
    def insights(self) -> InsightsResourceWithStreamingResponse:
        return InsightsResourceWithStreamingResponse(self._transactions.insights)


class AsyncTransactionsResourceWithStreamingResponse:
    def __init__(self, transactions: AsyncTransactionsResource) -> None:
        self._transactions = transactions

        self.retrieve = async_to_streamed_response_wrapper(
            transactions.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            transactions.list,
        )
        self.categorize = async_to_streamed_response_wrapper(
            transactions.categorize,
        )

    @cached_property
    def recurring(self) -> AsyncRecurringResourceWithStreamingResponse:
        return AsyncRecurringResourceWithStreamingResponse(self._transactions.recurring)

    @cached_property
    def insights(self) -> AsyncInsightsResourceWithStreamingResponse:
        return AsyncInsightsResourceWithStreamingResponse(self._transactions.insights)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["AccountRetrieveResponse"]


class AccountRetrieveResponse(BaseModel):
    projected_cash_flow: Optional[object] = FieldInfo(alias="projectedCashFlow", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["TransactionCategorizeResponse", "MerchantDetails"]


class MerchantDetails(BaseModel):
    """Detailed information about a merchant associated with a transaction."""

    address: Optional[object] = None


class TransactionCategorizeResponse(BaseModel):
    location: Optional[object] = None
    """Geographic location details for a transaction."""

    merchant_details: Optional[MerchantDetails] = FieldInfo(alias="merchantDetails", default=None)
    """Detailed information about a merchant associated with a transaction."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .tools import (
    ToolsResource,
    AsyncToolsResource,
    ToolsResourceWithRawResponse,
    AsyncToolsResourceWithRawResponse,
    ToolsResourceWithStreamingResponse,
    AsyncToolsResourceWithStreamingResponse,
)
from .advisor import (
    AdvisorResource,
    AsyncAdvisorResource,
    AdvisorResourceWithRawResponse,
    AsyncAdvisorResourceWithRawResponse,
    AdvisorResourceWithStreamingResponse,
    AsyncAdvisorResourceWithStreamingResponse,
)

__all__ = [
    "ToolsResource",
    "AsyncToolsResource",
    "ToolsResourceWithRawResponse",
    "AsyncToolsResourceWithRawResponse",
    "ToolsResourceWithStreamingResponse",
    "AsyncToolsResourceWithStreamingResponse",
    "AdvisorResource",
    "AsyncAdvisorResource",
    "AdvisorResourceWithRawResponse",
    "AsyncAdvisorResourceWithRawResponse",
    "AdvisorResourceWithStreamingResponse",
    "AsyncAdvisorResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .simulate_run_advanced_params import SimulateRunAdvancedParams as SimulateRunAdvancedParams
from .simulate_run_standard_response import SimulateRunStandardResponse as SimulateRunStandardResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .nfts import (
    NFTsResource,
    AsyncNFTsResource,
    NFTsResourceWithRawResponse,
    AsyncNFTsResourceWithRawResponse,
    NFTsResourceWithStreamingResponse,
    AsyncNFTsResourceWithStreamingResponse,
)
from .web3 import (
    Web3Resource,
    AsyncWeb3Resource,
    Web3ResourceWithRawResponse,
    AsyncWeb3ResourceWithRawResponse,
    Web3ResourceWithStreamingResponse,
    AsyncWeb3ResourceWithStreamingResponse,
)
from .wallets import (
    WalletsResource,
    AsyncWalletsResource,
    WalletsResourceWithRawResponse,
    AsyncWalletsResourceWithRawResponse,
    WalletsResourceWithStreamingResponse,
    AsyncWalletsResourceWithStreamingResponse,
)

__all__ = [
    "WalletsResource",
    "AsyncWalletsResource",
    "WalletsResourceWithRawResponse",
    "AsyncWalletsResourceWithRawResponse",
    "WalletsResourceWithStreamingResponse",
    "AsyncWalletsResourceWithStreamingResponse",
    "NFTsResource",
    "AsyncNFTsResource",
    "NFTsResourceWithRawResponse",
    "AsyncNFTsResourceWithRawResponse",
    "NFTsResourceWithStreamingResponse",
    "AsyncNFTsResourceWithStreamingResponse",
    "Web3Resource",
    "AsyncWeb3Resource",
    "Web3ResourceWithRawResponse",
    "AsyncWeb3ResourceWithRawResponse",
    "Web3ResourceWithStreamingResponse",
    "AsyncWeb3ResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .nfts import (
    NFTsResource,
    AsyncNFTsResource,
    NFTsResourceWithRawResponse,
    AsyncNFTsResourceWithRawResponse,
    NFTsResourceWithStreamingResponse,
    AsyncNFTsResourceWithStreamingResponse,
)
from .wallets import (
    WalletsResource,
    AsyncWalletsResource,
    WalletsResourceWithRawResponse,
    AsyncWalletsResourceWithRawResponse,
    WalletsResourceWithStreamingResponse,
    AsyncWalletsResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["Web3Resource", "AsyncWeb3Resource"]


class Web3Resource(SyncAPIResource):
    @cached_property
    def wallets(self) -> WalletsResource:
        return WalletsResource(self._client)

    @cached_property
    def nfts(self) -> NFTsResource:
        return NFTsResource(self._client)

    @cached_property
    def with_raw_response(self) -> Web3ResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/diplomat-bit/aibank#accessing-raw-response-data-eg-headers
        """
        return Web3ResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> Web3ResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/diplomat-bit/aibank#with_streaming_response
        """
        return Web3ResourceWithStreamingResponse(self)


class AsyncWeb3Resource(AsyncAPIResource):
    @cached_property
    def wallets(self) -> AsyncWalletsResource:
        return AsyncWalletsResource(self._client)

    @cached_property
    def nfts(self) -> AsyncNFTsResource:
        return AsyncNFTsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncWeb3ResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/diplomat-bit/aibank#accessing-raw-response-data-eg-headers
        """
        return AsyncWeb3ResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncWeb3ResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/diplomat-bit/aibank#with_streaming_response
        """
        return AsyncWeb3ResourceWithStreamingResponse(self)


class Web3ResourceWithRawResponse:
    def __init__(self, web3: Web3Resource) -> None:
        self._web3 = web3

    @cached_property
    def wallets(self) -> WalletsResourceWithRawResponse:
        return WalletsResourceWithRawResponse(self._web3.wallets)

    @cached_property
    def nfts(self) -> NFTsResourceWithRawResponse:
        return NFTsResourceWithRawResponse(self._web3.nfts)


class AsyncWeb3ResourceWithRawResponse:
    def __init__(self, web3: AsyncWeb3Resource) -> None:
        self._web3 = web3

    @cached_property
    def wallets(self) -> AsyncWalletsResourceWithRawResponse:
        return AsyncWalletsResourceWithRawResponse(self._web3.wallets)

    @cached_property
    def nfts(self) -> AsyncNFTsResourceWithRawResponse:
        return AsyncNFTsResourceWithRawResponse(self._web3.nfts)


class Web3ResourceWithStreamingResponse:
    def __init__(self, web3: Web3Resource) -> None:
        self._web3 = web3

    @cached_property
    def wallets(self) -> WalletsResourceWithStreamingResponse:
        return WalletsResourceWithStreamingResponse(self._web3.wallets)

    @cached_property
    def nfts(self) -> NFTsResourceWithStreamingResponse:
        return NFTsResourceWithStreamingResponse(self._web3.nfts)


class AsyncWeb3ResourceWithStreamingResponse:
    def __init__(self, web3: AsyncWeb3Resource) -> None:
        self._web3 = web3

    @cached_property
    def wallets(self) -> AsyncWalletsResourceWithStreamingResponse:
        return AsyncWalletsResourceWithStreamingResponse(self._web3.wallets)

    @cached_property
    def nfts(self) -> AsyncNFTsResourceWithStreamingResponse:
        return AsyncNFTsResourceWithStreamingResponse(self._web3.nfts)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["StatementListParams"]


class StatementListParams(TypedDict, total=False):
    format: str
    """Desired format for the statement.

    Use 'application/json' Accept header for download links.
    """

    month: int
    """Month for the statement (1-12)."""

    year: int
    """Year for the statement."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from jocall3 import Jocall3, AsyncJocall3
from tests.utils import assert_matches_type
from jocall3.types.corporate import (
    CardFreezeResponse,
    CardIssueVirtualResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCards:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Jocall3) -> None:
        card = client.corporate.cards.list()
        assert_matches_type(object, card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Jocall3) -> None:
        card = client.corporate.cards.list(
            limit=0,
            offset=0,
        )
        assert_matches_type(object, card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Jocall3) -> None:
        response = client.corporate.cards.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        card = response.parse()
        assert_matches_type(object, card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Jocall3) -> None:
        with client.corporate.cards.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            card = response.parse()
            assert_matches_type(object, card, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_freeze(self, client: Jocall3) -> None:
        card = client.corporate.cards.freeze(
            "corp_card_xyz987654",
        )
        assert_matches_type(CardFreezeResponse, card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_freeze(self, client: Jocall3) -> None:
        response = client.corporate.cards.with_raw_response.freeze(
            "corp_card_xyz987654",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        card = response.parse()
        assert_matches_type(CardFreezeResponse, card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_freeze(self, client: Jocall3) -> None:
        with client.corporate.cards.with_streaming_response.freeze(
            "corp_card_xyz987654",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            card = response.parse()
            assert_matches_type(CardFreezeResponse, card, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_freeze(self, client: Jocall3) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `card_id` but received ''"):
            client.corporate.cards.with_raw_response.freeze(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_issue_virtual(self, client: Jocall3) -> None:
        card = client.corporate.cards.issue_virtual(
            controls={
                "atmWithdrawals": False,
                "contactlessPayments": False,
                "onlineTransactions": True,
                "internationalTransactions": False,
                "monthlyLimit": 1000,
                "dailyLimit": 500,
                "singleTransactionLimit": 200,
                "merchantCategoryRestrictions": ["Advertising"],
                "vendorRestrictions": ["Facebook Ads", "Google Ads"],
            },
        )
        assert_matches_type(CardIssueVirtualResponse, card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_issue_virtual(self, client: Jocall3) -> None:
        response = client.corporate.cards.with_raw_response.issue_virtual(
            controls={
                "atmWithdrawals": False,
                "contactlessPayments": False,
                "onlineTransactions": True,
                "internationalTransactions": False,
                "monthlyLimit": 1000,
                "dailyLimit": 500,
                "singleTransactionLimit": 200,
                "merchantCategoryRestrictions": ["Advertising"],
                "vendorRestrictions": ["Facebook Ads", "Google Ads"],
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        card = response.parse()
        assert_matches_type(CardIssueVirtualResponse, card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_issue_virtual(self, client: Jocall3) -> None:
        with client.corporate.cards.with_streaming_response.issue_virtual(
            controls={
                "atmWithdrawals": False,
                "contactlessPayments": False,
                "onlineTransactions": True,
                "internationalTransactions": False,
                "monthlyLimit": 1000,
                "dailyLimit": 500,
                "singleTransactionLimit": 200,
                "merchantCategoryRestrictions": ["Advertising"],
                "vendorRestrictions": ["Facebook Ads", "Google Ads"],
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            card = response.parse()
            assert_matches_type(CardIssueVirtualResponse, card, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncCards:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncJocall3) -> None:
        card = await async_client.corporate.cards.list()
        assert_matches_type(object, card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncJocall3) -> None:
        card = await async_client.corporate.cards.list(
            limit=0,
            offset=0,
        )
        assert_matches_type(object, card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncJocall3) -> None:
        response = await async_client.corporate.cards.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        card = await response.parse()
        assert_matches_type(object, card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncJocall3) -> None:
        async with async_client.corporate.cards.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            card = await response.parse()
            assert_matches_type(object, card, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_freeze(self, async_client: AsyncJocall3) -> None:
        card = await async_client.corporate.cards.freeze(
            "corp_card_xyz987654",
        )
        assert_matches_type(CardFreezeResponse, card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_freeze(self, async_client: AsyncJocall3) -> None:
        response = await async_client.corporate.cards.with_raw_response.freeze(
            "corp_card_xyz987654",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        card = await response.parse()
        assert_matches_type(CardFreezeResponse, card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_freeze(self, async_client: AsyncJocall3) -> None:
        async with async_client.corporate.cards.with_streaming_response.freeze(
            "corp_card_xyz987654",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            card = await response.parse()
            assert_matches_type(CardFreezeResponse, card, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_freeze(self, async_client: AsyncJocall3) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `card_id` but received ''"):
            await async_client.corporate.cards.with_raw_response.freeze(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_issue_virtual(self, async_client: AsyncJocall3) -> None:
        card = await async_client.corporate.cards.issue_virtual(
            controls={
                "atmWithdrawals": False,
                "contactlessPayments": False,
                "onlineTransactions": True,
                "internationalTransactions": False,
                "monthlyLimit": 1000,
                "dailyLimit": 500,
                "singleTransactionLimit": 200,
                "merchantCategoryRestrictions": ["Advertising"],
                "vendorRestrictions": ["Facebook Ads", "Google Ads"],
            },
        )
        assert_matches_type(CardIssueVirtualResponse, card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_issue_virtual(self, async_client: AsyncJocall3) -> None:
        response = await async_client.corporate.cards.with_raw_response.issue_virtual(
            controls={
                "atmWithdrawals": False,
                "contactlessPayments": False,
                "onlineTransactions": True,
                "internationalTransactions": False,
                "monthlyLimit": 1000,
                "dailyLimit": 500,
                "singleTransactionLimit": 200,
                "merchantCategoryRestrictions": ["Advertising"],
                "vendorRestrictions": ["Facebook Ads", "Google Ads"],
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        card = await response.parse()
        assert_matches_type(CardIssueVirtualResponse, card, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_issue_virtual(self, async_client: AsyncJocall3) -> None:
        async with async_client.corporate.cards.with_streaming_response.issue_virtual(
            controls={
                "atmWithdrawals": False,
                "contactlessPayments": False,
                "onlineTransactions": True,
                "internationalTransactions": False,
                "monthlyLimit": 1000,
                "dailyLimit": 500,
                "singleTransactionLimit": 200,
                "merchantCategoryRestrictions": ["Advertising"],
                "vendorRestrictions": ["Facebook Ads", "Google Ads"],
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            card = await response.parse()
            assert_matches_type(CardIssueVirtualResponse, card, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ....types.ai import incubator_generate_pitch_params
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options

__all__ = ["IncubatorResource", "AsyncIncubatorResource"]


class IncubatorResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> IncubatorResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/diplomat-bit/aibank#accessing-raw-response-data-eg-headers
        """
        return IncubatorResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> IncubatorResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/diplomat-bit/aibank#with_streaming_response
        """
        return IncubatorResourceWithStreamingResponse(self)

    def generate_pitch(
        self,
        *,
        financial_projections: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Submits a detailed business plan to the Quantum Weaver AI for rigorous analysis,
        market validation, and seed funding consideration. This initiates the AI-driven
        incubation journey, aiming to transform innovative ideas into commercially
        successful ventures.

        Args:
          financial_projections: Key financial metrics and projections for the next 3-5 years.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/ai/incubator/pitch",
            body=maybe_transform(
                {"financial_projections": financial_projections},
                incubator_generate_pitch_params.IncubatorGeneratePitchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncIncubatorResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncIncubatorResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/diplomat-bit/aibank#accessing-raw-response-data-eg-headers
        """
        return AsyncIncubatorResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncIncubatorResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/diplomat-bit/aibank#with_streaming_response
        """
        return AsyncIncubatorResourceWithStreamingResponse(self)

    async def generate_pitch(
        self,
        *,
        financial_projections: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Submits a detailed business plan to the Quantum Weaver AI for rigorous analysis,
        market validation, and seed funding consideration. This initiates the AI-driven
        incubation journey, aiming to transform innovative ideas into commercially
        successful ventures.

        Args:
          financial_projections: Key financial metrics and projections for the next 3-5 years.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/ai/incubator/pitch",
            body=await async_maybe_transform(
                {"financial_projections": financial_projections},
                incubator_generate_pitch_params.IncubatorGeneratePitchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class IncubatorResourceWithRawResponse:
    def __init__(self, incubator: IncubatorResource) -> None:
        self._incubator = incubator

        self.generate_pitch = to_raw_response_wrapper(
            incubator.generate_pitch,
        )


class AsyncIncubatorResourceWithRawResponse:
    def __init__(self, incubator: AsyncIncubatorResource) -> None:
        self._incubator = incubator

        self.generate_pitch = async_to_raw_response_wrapper(
            incubator.generate_pitch,
        )


class IncubatorResourceWithStreamingResponse:
    def __init__(self, incubator: IncubatorResource) -> None:
        self._incubator = incubator

        self.generate_pitch = to_streamed_response_wrapper(
            incubator.generate_pitch,
        )


class AsyncIncubatorResourceWithStreamingResponse:
    def __init__(self, incubator: AsyncIncubatorResource) -> None:
        self._incubator = incubator

        self.generate_pitch = async_to_streamed_response_wrapper(
            incubator.generate_pitch,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["MeUpdateResponse", "Preferences"]


class Preferences(BaseModel):
    """User's personalized preferences for the platform."""

    notification_channels: Optional[object] = FieldInfo(alias="notificationChannels", default=None)
    """Preferred channels for receiving notifications."""


class MeUpdateResponse(BaseModel):
    address: Optional[object] = None

    preferences: Optional[Preferences] = None
    """User's personalized preferences for the platform."""

    security_status: Optional[object] = FieldInfo(alias="securityStatus", default=None)
    """Security-related status for the user account."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any, Dict, Mapping, cast
from typing_extensions import Self, Literal, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import Jocall3Error, APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

if TYPE_CHECKING:
    from .resources import ai, web3, users, accounts, payments, corporate, marketplace, transactions, sustainability
    from .resources.ai.ai import AIResource, AsyncAIResource
    from .resources.web3.web3 import Web3Resource, AsyncWeb3Resource
    from .resources.users.users import UsersResource, AsyncUsersResource
    from .resources.accounts.accounts import AccountsResource, AsyncAccountsResource
    from .resources.payments.payments import PaymentsResource, AsyncPaymentsResource
    from .resources.corporate.corporate import CorporateResource, AsyncCorporateResource
    from .resources.marketplace.marketplace import MarketplaceResource, AsyncMarketplaceResource
    from .resources.transactions.transactions import TransactionsResource, AsyncTransactionsResource
    from .resources.sustainability.sustainability import SustainabilityResource, AsyncSustainabilityResource

__all__ = [
    "ENVIRONMENTS",
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "Jocall3",
    "AsyncJocall3",
    "Client",
    "AsyncClient",
]

ENVIRONMENTS: Dict[str, str] = {
    "production": "https://75975599-8fdc-4274-8701-05fc0b8089cc.mock.pstmn.io",
    "sandbox": "https://75975599-8fdc-4274-8701-05fc0b8089cc.mock.pstmn.io",
    "gemini_direct": "https://generativelanguage.googleapis.com/v1beta",
}


class Jocall3(SyncAPIClient):
    # client options
    api_key: str
    gemini_api_key: str

    _environment: Literal["production", "sandbox", "gemini_direct"] | NotGiven

    def __init__(
        self,
        *,
        api_key: str | None = None,
        gemini_api_key: str | None = None,
        environment: Literal["production", "sandbox", "gemini_direct"] | NotGiven = not_given,
        base_url: str | httpx.URL | None | NotGiven = not_given,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Jocall3 client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `JOCALL3_API_KEY`
        - `gemini_api_key` from `GEMINI_API_KEY`
        """
        if api_key is None:
            api_key = os.environ.get("JOCALL3_API_KEY")
        if api_key is None:
            raise Jocall3Error(
                "The api_key client option must be set either by passing api_key to the client or by setting the JOCALL3_API_KEY environment variable"
            )
        self.api_key = api_key

        if gemini_api_key is None:
            gemini_api_key = os.environ.get("GEMINI_API_KEY")
        if gemini_api_key is None:
            raise Jocall3Error(
                "The gemini_api_key client option must be set either by passing gemini_api_key to the client or by setting the GEMINI_API_KEY environment variable"
            )
        self.gemini_api_key = gemini_api_key

        self._environment = environment

        base_url_env = os.environ.get("JOCALL3_BASE_URL")
        if is_given(base_url) and base_url is not None:
            # cast required because mypy doesn't understand the type narrowing
            base_url = cast("str | httpx.URL", base_url)  # pyright: ignore[reportUnnecessaryCast]
        elif is_given(environment):
            if base_url_env and base_url is not None:
                raise ValueError(
                    "Ambiguous URL; The `JOCALL3_BASE_URL` env var and the `environment` argument are given. If you want to use the environment, you must pass base_url=None",
                )

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc
        elif base_url_env is not None:
            base_url = base_url_env
        else:
            self._environment = environment = "production"

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def users(self) -> UsersResource:
        from .resources.users import UsersResource

        return UsersResource(self)

    @cached_property
    def accounts(self) -> AccountsResource:
        from .resources.accounts import AccountsResource

        return AccountsResource(self)

    @cached_property
    def transactions(self) -> TransactionsResource:
        from .resources.transactions import TransactionsResource

        return TransactionsResource(self)

    @cached_property
    def ai(self) -> AIResource:
        from .resources.ai import AIResource

        return AIResource(self)

    @cached_property
    def corporate(self) -> CorporateResource:
        from .resources.corporate import CorporateResource

        return CorporateResource(self)

    @cached_property
    def web3(self) -> Web3Resource:
        from .resources.web3 import Web3Resource

        return Web3Resource(self)

    @cached_property
    def payments(self) -> PaymentsResource:
        from .resources.payments import PaymentsResource

        return PaymentsResource(self)

    @cached_property
    def sustainability(self) -> SustainabilityResource:
        from .resources.sustainability import SustainabilityResource

        return SustainabilityResource(self)

    @cached_property
    def marketplace(self) -> MarketplaceResource:
        from .resources.marketplace import MarketplaceResource

        return MarketplaceResource(self)

    @cached_property
    def with_raw_response(self) -> Jocall3WithRawResponse:
        return Jocall3WithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> Jocall3WithStreamedResponse:
        return Jocall3WithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        return {**self._bearer_auth, **self._gemini_header}

    @property
    def _bearer_auth(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    def _gemini_header(self) -> dict[str, str]:
        gemini_api_key = self.gemini_api_key
        return {"x-goog-api-key": gemini_api_key}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        gemini_api_key: str | None = None,
        environment: Literal["production", "sandbox", "gemini_direct"] | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            gemini_api_key=gemini_api_key or self.gemini_api_key,
            base_url=base_url or self.base_url,
            environment=environment or self._environment,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncJocall3(AsyncAPIClient):
    # client options
    api_key: str
    gemini_api_key: str

    _environment: Literal["production", "sandbox", "gemini_direct"] | NotGiven

    def __init__(
        self,
        *,
        api_key: str | None = None,
        gemini_api_key: str | None = None,
        environment: Literal["production", "sandbox", "gemini_direct"] | NotGiven = not_given,
        base_url: str | httpx.URL | None | NotGiven = not_given,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncJocall3 client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `JOCALL3_API_KEY`
        - `gemini_api_key` from `GEMINI_API_KEY`
        """
        if api_key is None:
            api_key = os.environ.get("JOCALL3_API_KEY")
        if api_key is None:
            raise Jocall3Error(
                "The api_key client option must be set either by passing api_key to the client or by setting the JOCALL3_API_KEY environment variable"
            )
        self.api_key = api_key

        if gemini_api_key is None:
            gemini_api_key = os.environ.get("GEMINI_API_KEY")
        if gemini_api_key is None:
            raise Jocall3Error(
                "The gemini_api_key client option must be set either by passing gemini_api_key to the client or by setting the GEMINI_API_KEY environment variable"
            )
        self.gemini_api_key = gemini_api_key

        self._environment = environment

        base_url_env = os.environ.get("JOCALL3_BASE_URL")
        if is_given(base_url) and base_url is not None:
            # cast required because mypy doesn't understand the type narrowing
            base_url = cast("str | httpx.URL", base_url)  # pyright: ignore[reportUnnecessaryCast]
        elif is_given(environment):
            if base_url_env and base_url is not None:
                raise ValueError(
                    "Ambiguous URL; The `JOCALL3_BASE_URL` env var and the `environment` argument are given. If you want to use the environment, you must pass base_url=None",
                )

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc
        elif base_url_env is not None:
            base_url = base_url_env
        else:
            self._environment = environment = "production"

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def users(self) -> AsyncUsersResource:
        from .resources.users import AsyncUsersResource

        return AsyncUsersResource(self)

    @cached_property
    def accounts(self) -> AsyncAccountsResource:
        from .resources.accounts import AsyncAccountsResource

        return AsyncAccountsResource(self)

    @cached_property
    def transactions(self) -> AsyncTransactionsResource:
        from .resources.transactions import AsyncTransactionsResource

        return AsyncTransactionsResource(self)

    @cached_property
    def ai(self) -> AsyncAIResource:
        from .resources.ai import AsyncAIResource

        return AsyncAIResource(self)

    @cached_property
    def corporate(self) -> AsyncCorporateResource:
        from .resources.corporate import AsyncCorporateResource

        return AsyncCorporateResource(self)

    @cached_property
    def web3(self) -> AsyncWeb3Resource:
        from .resources.web3 import AsyncWeb3Resource

        return AsyncWeb3Resource(self)

    @cached_property
    def payments(self) -> AsyncPaymentsResource:
        from .resources.payments import AsyncPaymentsResource

        return AsyncPaymentsResource(self)

    @cached_property
    def sustainability(self) -> AsyncSustainabilityResource:
        from .resources.sustainability import AsyncSustainabilityResource

        return AsyncSustainabilityResource(self)

    @cached_property
    def marketplace(self) -> AsyncMarketplaceResource:
        from .resources.marketplace import AsyncMarketplaceResource

        return AsyncMarketplaceResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncJocall3WithRawResponse:
        return AsyncJocall3WithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncJocall3WithStreamedResponse:
        return AsyncJocall3WithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        return {**self._bearer_auth, **self._gemini_header}

    @property
    def _bearer_auth(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    def _gemini_header(self) -> dict[str, str]:
        gemini_api_key = self.gemini_api_key
        return {"x-goog-api-key": gemini_api_key}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        gemini_api_key: str | None = None,
        environment: Literal["production", "sandbox", "gemini_direct"] | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            gemini_api_key=gemini_api_key or self.gemini_api_key,
            base_url=base_url or self.base_url,
            environment=environment or self._environment,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class Jocall3WithRawResponse:
    _client: Jocall3

    def __init__(self, client: Jocall3) -> None:
        self._client = client

    @cached_property
    def users(self) -> users.UsersResourceWithRawResponse:
        from .resources.users import UsersResourceWithRawResponse

        return UsersResourceWithRawResponse(self._client.users)

    @cached_property
    def accounts(self) -> accounts.AccountsResourceWithRawResponse:
        from .resources.accounts import AccountsResourceWithRawResponse

        return AccountsResourceWithRawResponse(self._client.accounts)

    @cached_property
    def transactions(self) -> transactions.TransactionsResourceWithRawResponse:
        from .resources.transactions import TransactionsResourceWithRawResponse

        return TransactionsResourceWithRawResponse(self._client.transactions)

    @cached_property
    def ai(self) -> ai.AIResourceWithRawResponse:
        from .resources.ai import AIResourceWithRawResponse

        return AIResourceWithRawResponse(self._client.ai)

    @cached_property
    def corporate(self) -> corporate.CorporateResourceWithRawResponse:
        from .resources.corporate import CorporateResourceWithRawResponse

        return CorporateResourceWithRawResponse(self._client.corporate)

    @cached_property
    def web3(self) -> web3.Web3ResourceWithRawResponse:
        from .resources.web3 import Web3ResourceWithRawResponse

        return Web3ResourceWithRawResponse(self._client.web3)

    @cached_property
    def payments(self) -> payments.PaymentsResourceWithRawResponse:
        from .resources.payments import PaymentsResourceWithRawResponse

        return PaymentsResourceWithRawResponse(self._client.payments)

    @cached_property
    def sustainability(self) -> sustainability.SustainabilityResourceWithRawResponse:
        from .resources.sustainability import SustainabilityResourceWithRawResponse

        return SustainabilityResourceWithRawResponse(self._client.sustainability)

    @cached_property
    def marketplace(self) -> marketplace.MarketplaceResourceWithRawResponse:
        from .resources.marketplace import MarketplaceResourceWithRawResponse

        return MarketplaceResourceWithRawResponse(self._client.marketplace)


class AsyncJocall3WithRawResponse:
    _client: AsyncJocall3

    def __init__(self, client: AsyncJocall3) -> None:
        self._client = client

    @cached_property
    def users(self) -> users.AsyncUsersResourceWithRawResponse:
        from .resources.users import AsyncUsersResourceWithRawResponse

        return AsyncUsersResourceWithRawResponse(self._client.users)

    @cached_property
    def accounts(self) -> accounts.AsyncAccountsResourceWithRawResponse:
        from .resources.accounts import AsyncAccountsResourceWithRawResponse

        return AsyncAccountsResourceWithRawResponse(self._client.accounts)

    @cached_property
    def transactions(self) -> transactions.AsyncTransactionsResourceWithRawResponse:
        from .resources.transactions import AsyncTransactionsResourceWithRawResponse

        return AsyncTransactionsResourceWithRawResponse(self._client.transactions)

    @cached_property
    def ai(self) -> ai.AsyncAIResourceWithRawResponse:
        from .resources.ai import AsyncAIResourceWithRawResponse

        return AsyncAIResourceWithRawResponse(self._client.ai)

    @cached_property
    def corporate(self) -> corporate.AsyncCorporateResourceWithRawResponse:
        from .resources.corporate import AsyncCorporateResourceWithRawResponse

        return AsyncCorporateResourceWithRawResponse(self._client.corporate)

    @cached_property
    def web3(self) -> web3.AsyncWeb3ResourceWithRawResponse:
        from .resources.web3 import AsyncWeb3ResourceWithRawResponse

        return AsyncWeb3ResourceWithRawResponse(self._client.web3)

    @cached_property
    def payments(self) -> payments.AsyncPaymentsResourceWithRawResponse:
        from .resources.payments import AsyncPaymentsResourceWithRawResponse

        return AsyncPaymentsResourceWithRawResponse(self._client.payments)

    @cached_property
    def sustainability(self) -> sustainability.AsyncSustainabilityResourceWithRawResponse:
        from .resources.sustainability import AsyncSustainabilityResourceWithRawResponse

        return AsyncSustainabilityResourceWithRawResponse(self._client.sustainability)

    @cached_property
    def marketplace(self) -> marketplace.AsyncMarketplaceResourceWithRawResponse:
        from .resources.marketplace import AsyncMarketplaceResourceWithRawResponse

        return AsyncMarketplaceResourceWithRawResponse(self._client.marketplace)


class Jocall3WithStreamedResponse:
    _client: Jocall3

    def __init__(self, client: Jocall3) -> None:
        self._client = client

    @cached_property
    def users(self) -> users.UsersResourceWithStreamingResponse:
        from .resources.users import UsersResourceWithStreamingResponse

        return UsersResourceWithStreamingResponse(self._client.users)

    @cached_property
    def accounts(self) -> accounts.AccountsResourceWithStreamingResponse:
        from .resources.accounts import AccountsResourceWithStreamingResponse

        return AccountsResourceWithStreamingResponse(self._client.accounts)

    @cached_property
    def transactions(self) -> transactions.TransactionsResourceWithStreamingResponse:
        from .resources.transactions import TransactionsResourceWithStreamingResponse

        return TransactionsResourceWithStreamingResponse(self._client.transactions)

    @cached_property
    def ai(self) -> ai.AIResourceWithStreamingResponse:
        from .resources.ai import AIResourceWithStreamingResponse

        return AIResourceWithStreamingResponse(self._client.ai)

    @cached_property
    def corporate(self) -> corporate.CorporateResourceWithStreamingResponse:
        from .resources.corporate import CorporateResourceWithStreamingResponse

        return CorporateResourceWithStreamingResponse(self._client.corporate)

    @cached_property
    def web3(self) -> web3.Web3ResourceWithStreamingResponse:
        from .resources.web3 import Web3ResourceWithStreamingResponse

        return Web3ResourceWithStreamingResponse(self._client.web3)

    @cached_property
    def payments(self) -> payments.PaymentsResourceWithStreamingResponse:
        from .resources.payments import PaymentsResourceWithStreamingResponse

        return PaymentsResourceWithStreamingResponse(self._client.payments)

    @cached_property
    def sustainability(self) -> sustainability.SustainabilityResourceWithStreamingResponse:
        from .resources.sustainability import SustainabilityResourceWithStreamingResponse

        return SustainabilityResourceWithStreamingResponse(self._client.sustainability)

    @cached_property
    def marketplace(self) -> marketplace.MarketplaceResourceWithStreamingResponse:
        from .resources.marketplace import MarketplaceResourceWithStreamingResponse

        return MarketplaceResourceWithStreamingResponse(self._client.marketplace)


class AsyncJocall3WithStreamedResponse:
    _client: AsyncJocall3

    def __init__(self, client: AsyncJocall3) -> None:
        self._client = client

    @cached_property
    def users(self) -> users.AsyncUsersResourceWithStreamingResponse:
        from .resources.users import AsyncUsersResourceWithStreamingResponse

        return AsyncUsersResourceWithStreamingResponse(self._client.users)

    @cached_property
    def accounts(self) -> accounts.AsyncAccountsResourceWithStreamingResponse:
        from .resources.accounts import AsyncAccountsResourceWithStreamingResponse

        return AsyncAccountsResourceWithStreamingResponse(self._client.accounts)

    @cached_property
    def transactions(self) -> transactions.AsyncTransactionsResourceWithStreamingResponse:
        from .resources.transactions import AsyncTransactionsResourceWithStreamingResponse

        return AsyncTransactionsResourceWithStreamingResponse(self._client.transactions)

    @cached_property
    def ai(self) -> ai.AsyncAIResourceWithStreamingResponse:
        from .resources.ai import AsyncAIResourceWithStreamingResponse

        return AsyncAIResourceWithStreamingResponse(self._client.ai)

    @cached_property
    def corporate(self) -> corporate.AsyncCorporateResourceWithStreamingResponse:
        from .resources.corporate import AsyncCorporateResourceWithStreamingResponse

        return AsyncCorporateResourceWithStreamingResponse(self._client.corporate)

    @cached_property
    def web3(self) -> web3.AsyncWeb3ResourceWithStreamingResponse:
        from .resources.web3 import AsyncWeb3ResourceWithStreamingResponse

        return AsyncWeb3ResourceWithStreamingResponse(self._client.web3)

    @cached_property
    def payments(self) -> payments.AsyncPaymentsResourceWithStreamingResponse:
        from .resources.payments import AsyncPaymentsResourceWithStreamingResponse

        return AsyncPaymentsResourceWithStreamingResponse(self._client.payments)

    @cached_property
    def sustainability(self) -> sustainability.AsyncSustainabilityResourceWithStreamingResponse:
        from .resources.sustainability import AsyncSustainabilityResourceWithStreamingResponse

        return AsyncSustainabilityResourceWithStreamingResponse(self._client.sustainability)

    @cached_property
    def marketplace(self) -> marketplace.AsyncMarketplaceResourceWithStreamingResponse:
        from .resources.marketplace import AsyncMarketplaceResourceWithStreamingResponse

        return AsyncMarketplaceResourceWithStreamingResponse(self._client.marketplace)


Client = Jocall3

AsyncClient = AsyncJocall3



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .cards import (
    CardsResource,
    AsyncCardsResource,
    CardsResourceWithRawResponse,
    AsyncCardsResourceWithRawResponse,
    CardsResourceWithStreamingResponse,
    AsyncCardsResourceWithStreamingResponse,
)
from .controls import (
    ControlsResource,
    AsyncControlsResource,
    ControlsResourceWithRawResponse,
    AsyncControlsResourceWithRawResponse,
    ControlsResourceWithStreamingResponse,
    AsyncControlsResourceWithStreamingResponse,
)

__all__ = [
    "ControlsResource",
    "AsyncControlsResource",
    "ControlsResourceWithRawResponse",
    "AsyncControlsResourceWithRawResponse",
    "ControlsResourceWithStreamingResponse",
    "AsyncControlsResourceWithStreamingResponse",
    "CardsResource",
    "AsyncCardsResource",
    "CardsResourceWithRawResponse",
    "AsyncCardsResourceWithRawResponse",
    "CardsResourceWithStreamingResponse",
    "AsyncCardsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .risk import (
    RiskResource,
    AsyncRiskResource,
    RiskResourceWithRawResponse,
    AsyncRiskResourceWithRawResponse,
    RiskResourceWithStreamingResponse,
    AsyncRiskResourceWithStreamingResponse,
)
from .fraud import (
    FraudResource,
    AsyncFraudResource,
    FraudResourceWithRawResponse,
    AsyncFraudResourceWithRawResponse,
    FraudResourceWithStreamingResponse,
    AsyncFraudResourceWithStreamingResponse,
)

__all__ = [
    "FraudResource",
    "AsyncFraudResource",
    "FraudResourceWithRawResponse",
    "AsyncFraudResourceWithRawResponse",
    "FraudResourceWithStreamingResponse",
    "AsyncFraudResourceWithStreamingResponse",
    "RiskResource",
    "AsyncRiskResource",
    "RiskResourceWithRawResponse",
    "AsyncRiskResourceWithRawResponse",
    "RiskResourceWithStreamingResponse",
    "AsyncRiskResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["SimulateRunAdvancedParams"]


class SimulateRunAdvancedParams(TypedDict, total=False):
    global_economic_factors: Annotated[object, PropertyInfo(alias="globalEconomicFactors")]
    """Optional: Global economic conditions to apply to all scenarios."""

    personal_assumptions: Annotated[object, PropertyInfo(alias="personalAssumptions")]
    """Optional: Personal financial assumptions to override defaults."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .oracle.oracle import (
    OracleResource,
    AsyncOracleResource,
    OracleResourceWithRawResponse,
    AsyncOracleResourceWithRawResponse,
    OracleResourceWithStreamingResponse,
    AsyncOracleResourceWithStreamingResponse,
)
from .advisor.advisor import (
    AdvisorResource,
    AsyncAdvisorResource,
    AdvisorResourceWithRawResponse,
    AsyncAdvisorResourceWithRawResponse,
    AdvisorResourceWithStreamingResponse,
    AsyncAdvisorResourceWithStreamingResponse,
)
from .incubator.incubator import (
    IncubatorResource,
    AsyncIncubatorResource,
    IncubatorResourceWithRawResponse,
    AsyncIncubatorResourceWithRawResponse,
    IncubatorResourceWithStreamingResponse,
    AsyncIncubatorResourceWithStreamingResponse,
)

__all__ = ["AIResource", "AsyncAIResource"]


class AIResource(SyncAPIResource):
    @cached_property
    def advisor(self) -> AdvisorResource:
        return AdvisorResource(self._client)

    @cached_property
    def oracle(self) -> OracleResource:
        return OracleResource(self._client)

    @cached_property
    def incubator(self) -> IncubatorResource:
        return IncubatorResource(self._client)

    @cached_property
    def with_raw_response(self) -> AIResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/diplomat-bit/aibank#accessing-raw-response-data-eg-headers
        """
        return AIResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AIResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/diplomat-bit/aibank#with_streaming_response
        """
        return AIResourceWithStreamingResponse(self)


class AsyncAIResource(AsyncAPIResource):
    @cached_property
    def advisor(self) -> AsyncAdvisorResource:
        return AsyncAdvisorResource(self._client)

    @cached_property
    def oracle(self) -> AsyncOracleResource:
        return AsyncOracleResource(self._client)

    @cached_property
    def incubator(self) -> AsyncIncubatorResource:
        return AsyncIncubatorResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncAIResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/diplomat-bit/aibank#accessing-raw-response-data-eg-headers
        """
        return AsyncAIResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAIResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/diplomat-bit/aibank#with_streaming_response
        """
        return AsyncAIResourceWithStreamingResponse(self)


class AIResourceWithRawResponse:
    def __init__(self, ai: AIResource) -> None:
        self._ai = ai

    @cached_property
    def advisor(self) -> AdvisorResourceWithRawResponse:
        return AdvisorResourceWithRawResponse(self._ai.advisor)

    @cached_property
    def oracle(self) -> OracleResourceWithRawResponse:
        return OracleResourceWithRawResponse(self._ai.oracle)

    @cached_property
    def incubator(self) -> IncubatorResourceWithRawResponse:
        return IncubatorResourceWithRawResponse(self._ai.incubator)


class AsyncAIResourceWithRawResponse:
    def __init__(self, ai: AsyncAIResource) -> None:
        self._ai = ai

    @cached_property
    def advisor(self) -> AsyncAdvisorResourceWithRawResponse:
        return AsyncAdvisorResourceWithRawResponse(self._ai.advisor)

    @cached_property
    def oracle(self) -> AsyncOracleResourceWithRawResponse:
        return AsyncOracleResourceWithRawResponse(self._ai.oracle)

    @cached_property
    def incubator(self) -> AsyncIncubatorResourceWithRawResponse:
        return AsyncIncubatorResourceWithRawResponse(self._ai.incubator)


class AIResourceWithStreamingResponse:
    def __init__(self, ai: AIResource) -> None:
        self._ai = ai

    @cached_property
    def advisor(self) -> AdvisorResourceWithStreamingResponse:
        return AdvisorResourceWithStreamingResponse(self._ai.advisor)

    @cached_property
    def oracle(self) -> OracleResourceWithStreamingResponse:
        return OracleResourceWithStreamingResponse(self._ai.oracle)

    @cached_property
    def incubator(self) -> IncubatorResourceWithStreamingResponse:
        return IncubatorResourceWithStreamingResponse(self._ai.incubator)


class AsyncAIResourceWithStreamingResponse:
    def __init__(self, ai: AsyncAIResource) -> None:
        self._ai = ai

    @cached_property
    def advisor(self) -> AsyncAdvisorResourceWithStreamingResponse:
        return AsyncAdvisorResourceWithStreamingResponse(self._ai.advisor)

    @cached_property
    def oracle(self) -> AsyncOracleResourceWithStreamingResponse:
        return AsyncOracleResourceWithStreamingResponse(self._ai.oracle)

    @cached_property
    def incubator(self) -> AsyncIncubatorResourceWithStreamingResponse:
        return AsyncIncubatorResourceWithStreamingResponse(self._ai.incubator)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.payments import fx_get_rates_params
from ...types.payments.fx_get_rates_response import FxGetRatesResponse

__all__ = ["FxResource", "AsyncFxResource"]


class FxResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> FxResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/diplomat-bit/aibank#accessing-raw-response-data-eg-headers
        """
        return FxResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FxResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/diplomat-bit/aibank#with_streaming_response
        """
        return FxResourceWithStreamingResponse(self)

    def convert(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Executes an instant currency conversion between two currencies, either from a
        balance or into a specified account.
        """
        return self._post(
            "/payments/fx/convert",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def get_rates(
        self,
        *,
        base_currency: str | Omit = omit,
        forecast_days: int | Omit = omit,
        target_currency: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FxGetRatesResponse:
        """
        Retrieves current and AI-predicted future foreign exchange rates for a specified
        currency pair, including bid/ask spreads and historical volatility data for
        informed decisions.

        Args:
          base_currency: The base currency code (e.g., USD).

          forecast_days: Number of days into the future to provide an AI-driven prediction.

          target_currency: The target currency code (e.g., EUR).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/payments/fx/rates",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "base_currency": base_currency,
                        "forecast_days": forecast_days,
                        "target_currency": target_currency,
                    },
                    fx_get_rates_params.FxGetRatesParams,
                ),
            ),
            cast_to=FxGetRatesResponse,
        )


class AsyncFxResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncFxResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/diplomat-bit/aibank#accessing-raw-response-data-eg-headers
        """
        return AsyncFxResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFxResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/diplomat-bit/aibank#with_streaming_response
        """
        return AsyncFxResourceWithStreamingResponse(self)

    async def convert(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Executes an instant currency conversion between two currencies, either from a
        balance or into a specified account.
        """
        return await self._post(
            "/payments/fx/convert",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def get_rates(
        self,
        *,
        base_currency: str | Omit = omit,
        forecast_days: int | Omit = omit,
        target_currency: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FxGetRatesResponse:
        """
        Retrieves current and AI-predicted future foreign exchange rates for a specified
        currency pair, including bid/ask spreads and historical volatility data for
        informed decisions.

        Args:
          base_currency: The base currency code (e.g., USD).

          forecast_days: Number of days into the future to provide an AI-driven prediction.

          target_currency: The target currency code (e.g., EUR).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/payments/fx/rates",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "base_currency": base_currency,
                        "forecast_days": forecast_days,
                        "target_currency": target_currency,
                    },
                    fx_get_rates_params.FxGetRatesParams,
                ),
            ),
            cast_to=FxGetRatesResponse,
        )


class FxResourceWithRawResponse:
    def __init__(self, fx: FxResource) -> None:
        self._fx = fx

        self.convert = to_raw_response_wrapper(
            fx.convert,
        )
        self.get_rates = to_raw_response_wrapper(
            fx.get_rates,
        )


class AsyncFxResourceWithRawResponse:
    def __init__(self, fx: AsyncFxResource) -> None:
        self._fx = fx

        self.convert = async_to_raw_response_wrapper(
            fx.convert,
        )
        self.get_rates = async_to_raw_response_wrapper(
            fx.get_rates,
        )


class FxResourceWithStreamingResponse:
    def __init__(self, fx: FxResource) -> None:
        self._fx = fx

        self.convert = to_streamed_response_wrapper(
            fx.convert,
        )
        self.get_rates = to_streamed_response_wrapper(
            fx.get_rates,
        )


class AsyncFxResourceWithStreamingResponse:
    def __init__(self, fx: AsyncFxResource) -> None:
        self._fx = fx

        self.convert = async_to_streamed_response_wrapper(
            fx.convert,
        )
        self.get_rates = async_to_streamed_response_wrapper(
            fx.get_rates,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .devices import (
    DevicesResource,
    AsyncDevicesResource,
    DevicesResourceWithRawResponse,
    AsyncDevicesResourceWithRawResponse,
    DevicesResourceWithStreamingResponse,
    AsyncDevicesResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from .biometrics import (
    BiometricsResource,
    AsyncBiometricsResource,
    BiometricsResourceWithRawResponse,
    AsyncBiometricsResourceWithRawResponse,
    BiometricsResourceWithStreamingResponse,
    AsyncBiometricsResourceWithStreamingResponse,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....types.users import me_update_params
from ...._base_client import make_request_options
from ....types.users.me_update_response import MeUpdateResponse
from ....types.users.me_retrieve_response import MeRetrieveResponse

__all__ = ["MeResource", "AsyncMeResource"]


class MeResource(SyncAPIResource):
    @cached_property
    def devices(self) -> DevicesResource:
        return DevicesResource(self._client)

    @cached_property
    def biometrics(self) -> BiometricsResource:
        return BiometricsResource(self._client)

    @cached_property
    def with_raw_response(self) -> MeResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/diplomat-bit/aibank#accessing-raw-response-data-eg-headers
        """
        return MeResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MeResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/diplomat-bit/aibank#with_streaming_response
        """
        return MeResourceWithStreamingResponse(self)

    def retrieve(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeRetrieveResponse:
        """
        Fetches the complete and dynamically updated profile information for the
        currently authenticated user, encompassing personal details, security status,
        gamification level, loyalty points, and linked identity attributes.
        """
        return self._get(
            "/users/me",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MeRetrieveResponse,
        )

    def update(
        self,
        *,
        address: object | Omit = omit,
        preferences: me_update_params.Preferences | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeUpdateResponse:
        """
        Updates selected fields of the currently authenticated user's profile
        information.

        Args:
          preferences: User's personalized preferences for the platform.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._put(
            "/users/me",
            body=maybe_transform(
                {
                    "address": address,
                    "preferences": preferences,
                },
                me_update_params.MeUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MeUpdateResponse,
        )


class AsyncMeResource(AsyncAPIResource):
    @cached_property
    def devices(self) -> AsyncDevicesResource:
        return AsyncDevicesResource(self._client)

    @cached_property
    def biometrics(self) -> AsyncBiometricsResource:
        return AsyncBiometricsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncMeResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/diplomat-bit/aibank#accessing-raw-response-data-eg-headers
        """
        return AsyncMeResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMeResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/diplomat-bit/aibank#with_streaming_response
        """
        return AsyncMeResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeRetrieveResponse:
        """
        Fetches the complete and dynamically updated profile information for the
        currently authenticated user, encompassing personal details, security status,
        gamification level, loyalty points, and linked identity attributes.
        """
        return await self._get(
            "/users/me",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MeRetrieveResponse,
        )

    async def update(
        self,
        *,
        address: object | Omit = omit,
        preferences: me_update_params.Preferences | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MeUpdateResponse:
        """
        Updates selected fields of the currently authenticated user's profile
        information.

        Args:
          preferences: User's personalized preferences for the platform.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._put(
            "/users/me",
            body=await async_maybe_transform(
                {
                    "address": address,
                    "preferences": preferences,
                },
                me_update_params.MeUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MeUpdateResponse,
        )


class MeResourceWithRawResponse:
    def __init__(self, me: MeResource) -> None:
        self._me = me

        self.retrieve = to_raw_response_wrapper(
            me.retrieve,
        )
        self.update = to_raw_response_wrapper(
            me.update,
        )

    @cached_property
    def devices(self) -> DevicesResourceWithRawResponse:
        return DevicesResourceWithRawResponse(self._me.devices)

    @cached_property
    def biometrics(self) -> BiometricsResourceWithRawResponse:
        return BiometricsResourceWithRawResponse(self._me.biometrics)


class AsyncMeResourceWithRawResponse:
    def __init__(self, me: AsyncMeResource) -> None:
        self._me = me

        self.retrieve = async_to_raw_response_wrapper(
            me.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            me.update,
        )

    @cached_property
    def devices(self) -> AsyncDevicesResourceWithRawResponse:
        return AsyncDevicesResourceWithRawResponse(self._me.devices)

    @cached_property
    def biometrics(self) -> AsyncBiometricsResourceWithRawResponse:
        return AsyncBiometricsResourceWithRawResponse(self._me.biometrics)


class MeResourceWithStreamingResponse:
    def __init__(self, me: MeResource) -> None:
        self._me = me

        self.retrieve = to_streamed_response_wrapper(
            me.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            me.update,
        )

    @cached_property
    def devices(self) -> DevicesResourceWithStreamingResponse:
        return DevicesResourceWithStreamingResponse(self._me.devices)

    @cached_property
    def biometrics(self) -> BiometricsResourceWithStreamingResponse:
        return BiometricsResourceWithStreamingResponse(self._me.biometrics)


class AsyncMeResourceWithStreamingResponse:
    def __init__(self, me: AsyncMeResource) -> None:
        self._me = me

        self.retrieve = async_to_streamed_response_wrapper(
            me.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            me.update,
        )

    @cached_property
    def devices(self) -> AsyncDevicesResourceWithStreamingResponse:
        return AsyncDevicesResourceWithStreamingResponse(self._me.devices)

    @cached_property
    def biometrics(self) -> AsyncBiometricsResourceWithStreamingResponse:
        return AsyncBiometricsResourceWithStreamingResponse(self._me.biometrics)
